/*

Copyright (C) 2006, Arizona Software by Simon Bovet. All rights reserved.

*/

// var ProVocPlugIn = null;

var sourceLanguage = "";
var targetLanguage = "";
var sourceFontFamily = "";
var targetFontFamily = "";
var commentFontFamily = "";

var files = null;
var fileID = null;
var lessons = null;
var lessonID = null;
var direction;
var currentDirection = 0;
var commentDisplay;
var mode;
var retries;
var currentChoices;
var delayChoices;
var speed;
var fontSize;
var color;

var choices = 3;

var source = "";
var target = "";
var comment = "";
var solution;
var attempts;
var displayingComment = false;
var displayingFullSolution = false;
var displayingPreferences = false;
var delayingChoices = false;
var settingUp = true;
var wasHidden = false;

var currentTimeout = null;

function emptyLesson()
{
	if (ProVocPlugIn)
		return ProVocPlugIn.isLessonEmpty();
	else
		return true;
}

function setFileID(id)
{
	fileID = id;
	if (ProVocPlugIn) {
		fileID = ProVocPlugIn.setCurrentFileID(fileID);
		sourceLanguage = ProVocPlugIn.sourceLanguage();
		targetLanguage = ProVocPlugIn.targetLanguage();
	} else {
		sourceLanguage = "Français";
		targetLanguage = "English";
	}
	setPreferenceForKey(fileID, "fileID");
}

function selectFileID(id)
{
	var filePopup = document.getElementById("filePopup");
	filePopup.value = id;
	setFileID(settingUp ? id : filePopup.value);

	lessons = new Array();
	if (ProVocPlugIn) {
		var c = ProVocPlugIn.numberOfLessons();
		for (var i = 0; i < c; i++) {
			var name = i == 0 ? getLocalizedString("All lessons") : ProVocPlugIn.nameOfLessonAtIndex(i);
			var id = ProVocPlugIn.identifierOfLessonAtIndex(i);
			lessons.push({name: name, id: id});
		}
	} else
		lessons.push({name: "Dummy Lesson", id: ""});
	populateLessonPopup();
	var lessonPopup = document.getElementById("lessonPopup");
	lessonPopup.value = lessonID;
	var value = settingUp ? lessonID : lessonPopup.value;
	if (value == null || value === undefined)
		value = "";
	setLessonID(value);
	
	updateUI();
}

function setLessonID(id)
{
	if (!(id == null || id === undefined)) {
		lessonID = id;
		if (ProVocPlugIn)
			lessonID = ProVocPlugIn.setCurrentLessonID(lessonID);
		setPreferenceForKey(lessonID, "lessonID");
	}
}

function globalKey(key)
{
	return "ch.arizona-software.provoc.widget-" + key;
}

function localKey(key)
{
	return widget.identifier + "-" + key;
}

function preferenceForKey(key, defaultValue)
{
	var value = null;
	if (window.widget) {
		value = widget.preferenceForKey(localKey(key));
		if (value == null || value === undefined)
			value = widget.preferenceForKey(globalKey(key));
	}
	if (value == null || value === undefined)
		return defaultValue;
	else
		return value;
}

function setPreferenceForKey(value, key)
{
	if (value === undefined)
		return;
	if (window.widget) {
		widget.setPreferenceForKey(value, localKey(key));
		if (value != null)
			widget.setPreferenceForKey(value, globalKey(key));
	}
}

function loadPreferences()
{
	var id = preferenceForKey("fileID", "none");
	if (id)
		setFileID(id);
	id = preferenceForKey("lessonID", "none");
	if (!(id == null || id === undefined))
		setLessonID(id);

	direction = preferenceForKey("direction", "version");
	commentDisplay = preferenceForKey("commentDisplay", "always");
	mode = preferenceForKey("mode", "questions");
	retries = preferenceForKey("retries", 2);
	delayChoices = preferenceForKey("delayChoices", false);
	speed = preferenceForKey("speed", "medium");
	fontSize = preferenceForKey("fontSize", "medium");
	color = preferenceForKey("color", "black");
}

function deletePreferences()
{
	setPreferenceForKey(null, "fileID");
	setPreferenceForKey(null, "lessonID");
	setPreferenceForKey(null, "direction");
	setPreferenceForKey(null, "commentDisplay");
	setPreferenceForKey(null, "mode");
	setPreferenceForKey(null, "retries");
	setPreferenceForKey(null, "delayChoices");
	setPreferenceForKey(null, "speed");
	setPreferenceForKey(null, "fontSize");
	setPreferenceForKey(null, "color");
}

var n = 0;

function nextWord() {
	if (ProVocPlugIn) {
		currentDirection = ProVocPlugIn.chooseDirection(direction);
		ProVocPlugIn.chooseRandomWord();
		source = ProVocPlugIn.getWordSource();
		target = ProVocPlugIn.getWordTarget();
		comment = ProVocPlugIn.getWordComment();
		if (!source)
			source = "";
		if (!target)
			target = "";
		if (!comment)
			comment = "";
	} else {
		if (direction == "version" || direction == "both" && Math.random() >= 0.5)
			currentDirection = "version";
		else
			currentDirection = "theme";
		n++;
		source = "Source " + n;
		target = "Target " + n;
		if (n % 1 == 0)
			comment = "Comment " + n;
		else
			comment = "";
	}
		
	if (mode == "questions" || mode == "mcq") {
		askQuestion(false);
		if (mode == "questions")
			document.getElementById("answer").focus();
		animateElementVisibility(150);
	} else if (mode == "cards")
		showCardRecto(true);
}

function populateFilePopup()
{
	var popup = document.getElementById("filePopup");
	
	while (popup.hasChildNodes())
		popup.removeChild(popup.firstChild);
	
	var c = files.length;
	for (var i = 0; i < c; i++) {
		var file = files[i];
		var element = document.createElement("option");
		element.value = file.id;
		element.innerText = file.name;
		popup.appendChild(element);
	}
}

function populateLessonPopup()
{
	var popup = document.getElementById("lessonPopup");
	
	while (popup.hasChildNodes())
		popup.removeChild(popup.firstChild);
	
	var c = lessons.length;
	for (var i = 0; i < c; i++) {
		var lesson = lessons[i];
		var element = document.createElement("option");
		element.value = lesson.id;
		element.innerText = lesson.name;
		popup.appendChild(element);
	}
}

var elementVisibility = new Array;

function setElementByIdVisible(id, state)
{
	var visibility = elementVisibility[id];
	if (!visibility) {
		visibility = {previous: true, from: 0.0, now: 0.0};
		elementVisibility[id] = visibility;
	}
	visibility.next = state;
	visibility.to = state ? 1.0 : 0.0;
}

function updateElementVisibility()
{
	for (var id in elementVisibility) {
		var visibility = elementVisibility[id];
		if (visibility.next != visibility.previous) {
			var elem = document.getElementById(id);
			visibility.now = visibility.next ? 1.0 : 0.0;
			elem.style.opacity = visibility.now;
			visibility.previous = visibility.next;
		}
		visibility.from = visibility.now;
	}
	visibilityAnimationDidFinish();
}

function prepareElementVisibilityAnimation()
{
	var ok = false;
	for (var id in elementVisibility) {
		var visibility = elementVisibility[id];
		visibility.animate = visibility.next != visibility.previous;
		if (visibility.animate) {
			visibility.previous = visibility.next;
			visibility.from = visibility.now;
			ok = true;
		}
	}
	return ok;
}

var visibilityAnimation = {duration:0, starttime:0, timer:null}
var afterVisibilityAnimationCommands = new Array;

function visibilityAnimationDidFinish()
{
	var what
	while (what = afterVisibilityAnimationCommands.shift())
		setTimeout(what.command, what.delay);
}

function animateVisibility()
{
	var time = (new Date).getTime();
	var t = clamp((time - visibilityAnimation.starttime) / visibilityAnimation.duration, 0.0, 1.0);
	
	if (t >= 1.0) {
		clearInterval (visibilityAnimation.timer);
		visibilityAnimation.timer = null;
	}
	
	var ease = 0.5 - (0.5 * Math.cos(Math.PI * t));
	for (var id in elementVisibility) {
		var visibility = elementVisibility[id];
		if (visibility.animate) {
			visibility.now = interpolation(visibility.from, visibility.to, ease);
			document.getElementById(id).style.opacity = visibility.now;
		}
	}
	
	if (t >= 1.0)
		visibilityAnimationDidFinish();
}

function finishAnimateVisibility()
{
	if (visibilityAnimation.timer) {
		visibilityAnimation.starttime = 0;
		animateVisibility();
	}
}

function animateElementVisibility(duration)
{
	finishAnimateVisibility();
	if (prepareElementVisibilityAnimation()) {
		var starttime = (new Date).getTime();
		visibilityAnimation.duration = duration;
		visibilityAnimation.starttime = starttime;
		visibilityAnimation.timer = setInterval("animateVisibility()", 13);
	} else
		visibilityAnimationDidFinish();
}

function doAfterAnimateVisibility(command, delay)
{
	afterVisibilityAnimationCommands.push({command:command, delay:delay});
}

var cardTimeout = null;
var cardNextAction = null;
var cardNextActionDelay = 0;
var cardActionPaused = false;

function cancelCardAction()
{
	if (cardTimeout) {
		clearTimeout(cardTimeout);
		cardTimeout = null;
		cardNextAction = null;
	}
}

function cardAction(command, delay)
{
	cancelCardAction();
	cardNextAction = command;
	cardNextActionDelay = delay;
	cardTimeout = setTimeout(command, delay);
}

function performCardAction()
{
	if (cardNextAction) {
		finishAnimateVisibility();
		var action = cardNextAction;
		cancelCardAction();
		setTimeout(action, 0);
	}
}

function pauseCardAction()
{
	if (!cardActionPaused && cardTimeout) {
		clearTimeout(cardTimeout);
		cardTimeout = null;
		cardActionPaused = true;
	}
}

function resumeCardAction()
{
	if (cardActionPaused) {
		cardActionPaused = false;
		cardAction(cardNextAction, cardNextActionDelay);
	}
}

function cardRectoDelay()
{
	var delay = 1000 * 60;
	switch (speed) {
		case "slow":
			delay = 4500;
			break;
		case "medium":
			delay = 2500;
			break;
		case "fast":
			delay = 1500;
			break;
	}
	return delay;
}

function cardVersoDelay()
{
	var delay = 1000 * 180;
	switch (speed) {
		case "slow":
			delay = 8000;
			break;
		case "medium":
			delay = 4000;
			break;
		case "fast":
			delay = 2500;
			break;
	}
	return delay;
}

function showCardRecto(animate)
{
	var willDisplayComment = comment.length > 0 && commentDisplay != "never";
	document.getElementById("cardWithComment").style.display = willDisplayComment ? "block" : "none";
	document.getElementById("cardWithoutComment").style.display = !willDisplayComment ? "block" : "none";
	displayingComment = commentDisplay == "always";
	if (currentDirection == "version") {
		document.getElementById("cardQuestion").innerText = source;
		document.getElementById("cardQuestion2").innerText = source;
		document.getElementById("cardAnswer").innerText = target;
		document.getElementById("cardAnswer2").innerText = target;
		if (commentDisplay == "source")
			displayingComment = true;
	} else {
		document.getElementById("cardQuestion").innerText = target;
		document.getElementById("cardQuestion2").innerText = target;
		document.getElementById("cardAnswer").innerText = source;
		document.getElementById("cardAnswer2").innerText = source;
		if (commentDisplay == "target")
			displayingComment = true;
	}
	document.getElementById("cardComment").innerText = comment;
	updateFontSize();
	setElementByIdVisible("cardQuestion", true);
	setElementByIdVisible("cardQuestion2", true);
	setElementByIdVisible("cardAnswer", false);
	setElementByIdVisible("cardAnswer2", false);
	setElementByIdVisible("cardComment", displayingComment);
	cardAction("showCardVerso()", cardRectoDelay());
	if (animate)
		animateElementVisibility(150);
	else
		updateElementVisibility();
}

function showCardVerso()
{
	setElementByIdVisible("cardAnswer", true);
	setElementByIdVisible("cardAnswer2", true);
	setElementByIdVisible("cardComment", true);
	cardAction("showNextCard()", cardVersoDelay());
	animateElementVisibility(150);
}

function hideCardVerso()
{
	setElementByIdVisible("cardQuestion", false);
	setElementByIdVisible("cardQuestion2", false);
	setElementByIdVisible("cardAnswer", false);
	setElementByIdVisible("cardAnswer2", false);
	setElementByIdVisible("cardComment", false);
}

function showNextCard()
{
	hideCardVerso();
	doAfterAnimateVisibility("nextWord()", 350);
	animateElementVisibility(400);
}

function createChoices()
{
	answers = new Array;
	if (ProVocPlugIn) {
		currentChoices = ProVocPlugIn.chooseChoicesWithSolution(choices, solution);
		solution = ProVocPlugIn.indexOfSolution();
		for (var i = 0; i < currentChoices; i++)
			answers.push(ProVocPlugIn.choiceAtIndex(i));
	} else {
		currentChoices = choices;
		var sol = (n * n * 7) % choices;
		for (var i = 0; i < currentChoices; i++)
			if (i == sol) {
				answers.push(solution);
				solution = i;
			} else
				answers.push("Choice #" + i + " in " + (currentDirection == "version" ? sourceLanguage : targetLanguage));
	}
	
	var text = "";
	var altText = "";
	for (var i = 0; i < currentChoices; i++) {
		text += "<div class='mcqChoiceDiv'><a href='javascript:choice(" + i + ");' id='mcqChoice" + i + "' class='mcqChoice'></a></div>";
		altText += "<div class='mcqDoneChoiceDiv'><a href='javascript:choice(" + i + ");' id='mcqDoneChoice" + i + "' class='mcq" + (i == solution ? "Correct" : "Wrong") + "Choice'></a></div>";
	}
	document.getElementById("mcqChoices").innerHTML = "<div id='mcqChoiceList'>" + text + "</div>" +
														"<div id='mcqDoneChoiceList'>" + altText + "</div>";
	for (var i = 0; i < currentChoices; i++) {
		y = 50 + 30 * i + "px";
		var elem = document.getElementById("mcqChoice" + i);
		elem.innerText = answers[i];
		elem.style.top = y;
		elem = document.getElementById("mcqDoneChoice" + i);
		elem.innerText = answers[i];
		elem.style.top = y;
	}
}

function min(a, b)
{
	return a < b ? a : b;
}

function setElementFont(id, family)
{
	if (!family || family.length == 0)
		family = "Helvetica Neue";
	switch (fontSize) {
		case "small": size = 16; break;
		case "medium": size = 24; break;
		case "large": size = 32; break;
	}
	if (id == "cardQuestion" || id == "cardQuestion2" || id == "cardAnswer" || id == "cardAnswer2")
		switch (fontSize) {
			case "small": size = 24; break;
			case "medium": size = 32; break;
			case "large": size = 48; break;
		}
	if (id == "comment")
		size = min(size, 24);
	if (id == "mcq")
		size = min(size, 24);
	document.getElementById(id).style.font = 'bold ' + size + 'px "' + family + '"';
	
	if (id == "answer") {
		setElementFont("readOnlyAnswer", family);
		setElementFont("solution", family);
		setElementFont("mcq", family);
	}
	if (id == "cardQuestion")
		setElementFont("cardQuestion2", family);
	if (id == "cardAnswer")
		setElementFont("cardAnswer2", family);
}

function updateFontSize()
{
	if (currentDirection == "version") {
		setElementFont("question", sourceFontFamily);
		setElementFont("answer", targetFontFamily);
		setElementFont("cardQuestion", sourceFontFamily);
		setElementFont("cardAnswer", targetFontFamily);
	} else {
		setElementFont("question", targetFontFamily);
		setElementFont("answer", sourceFontFamily);
		setElementFont("cardQuestion", targetFontFamily);
		setElementFont("cardAnswer", sourceFontFamily);
	}
	setElementFont("comment", commentFontFamily);
	setElementFont("cardComment", commentFontFamily);
}

function askQuestion(display) {
	displayingComment = commentDisplay == "always";
	document.getElementById("mcq").style.display = "none";
	if (mode == "questions") {
		document.getElementById("writtenAnswer").style.display = "block";
		document.getElementById("answer").value = "";
	} else
		document.getElementById("writtenAnswer").style.display = "none";
	if (currentDirection == "version") {
		document.getElementById("question").innerText = source;
		solution = target;
		if (commentDisplay == "source")
			displayingComment = true;
	} else {
		document.getElementById("question").innerText = target;
		solution = source;
		if (commentDisplay == "target")
			displayingComment = true;
	}
	if (mode == "mcq") {
		setElementByIdVisible("mcqChoices", false);
		updateElementVisibility();
		createChoices();
		document.getElementById("mcqDoneChoiceList").style.display = "none";
		document.getElementById("mcqChoiceList").style.display = "block";
		document.getElementById("mcq").style.display = "block";
	}
	updateFontSize();
	updateUI();
	attempts = 0;
	document.getElementById("comment").innerText = comment;
	setElementByIdVisible("question", true);
	delayingChoices = delayChoices;
	setElementByIdVisible("mcqChoices", !delayingChoices);
	if (mode == "questions" && !displayingFullSolution)
		document.getElementById("verifyButton").style.display = "block";
	if (mode == "mcq")
		document.getElementById("displayButton").style.display = delayingChoices ? "block" : "none";
	setElementByIdVisible("commentDiv", displayingComment && comment.length > 0);
	if (display)
		updateElementVisibility();
}

function displayFullSolution()
{
	if (!displayingFullSolution) {
		if (mode == "questions") {
			document.getElementById("answer").style.display = "none";
			document.getElementById("readOnlyAnswer").value = document.getElementById("answer").value;
			document.getElementById("answer").value = "";
			document.getElementById("readOnlyAnswer").style.display = "block";
			document.getElementById("readOnlyAnswer").focus();
		}
		document.getElementById("verifyButton").style.display = "none";
		if (mode == "questions")
			document.getElementById("continueButton").style.display = "block";
		else
			document.getElementById("mcqContinueButton").style.display = "block";
		displayingFullSolution = true;
	}
}

function displayCommentIfNecessary()
{
	if (displayingComment || comment.length == 0 || commentDisplay == "never")
		return false;
	
	setElementByIdVisible("commentDiv", true);
	displayingComment = true;
	displayFullSolution();
	return true;
}

function clearAnswer()
{
	document.getElementById("answer").value = "";
}

function prepareForNextQuestion()
{
	setElementByIdVisible("question", false);
	setElementByIdVisible("commentDiv", false);
	setElementByIdVisible("mcqChoices", false);
	setElementByIdVisible("mcqMark", false);
	doAfterAnimateVisibility("clearAnswer()", 0);
	doAfterAnimateVisibility("nextWord()", 100);
}

var markAnimation = {start: 0, duration: 1000, timer: null};

function animateMark()
{
	var t = ((new Date).getTime() - markAnimation.start) / markAnimation.duration;
	var opacity = 0;
	var up = 0.1;
	var sustain = 0.25;
	if (t < up)
		opacity = t / up;
	else if (t < up - sustain)
		opacity = 1.0;
	else
		opacity = 1.0 - (t - up - sustain) / (1.0 - up - sustain);
	document.getElementById("mark").style.opacity = clamp(opacity, 0.0, 1.0);
	if (t >= 1) {
		clearInterval(markAnimation.timer);
		markAnimation.timer = null;
	}
}

function mark(correct)
{
	document.getElementById("mark").src = correct ? "Images/Correct.png" : "Images/Wrong.png";
	markAnimation.start = (new Date).getTime();
	markAnimation.duration = correct ? 2000 : 1000;
	if (!markAnimation.timer)
		markAnimation.timer = setInterval("animateMark()", 13);
}

function answerEquivalentToSolution(answer, solution) {
	if (ProVocPlugIn)
		return ProVocPlugIn.isAnswerEquivalentToSolution(answer, solution);
	else
		return answer.toLowerCase() == solution.toLowerCase();
}

function answerEqualToSolution(answer, solution) {
	if (ProVocPlugIn)
		return ProVocPlugIn.isAnswerEqualToSolution(answer, solution);
	else
		return answer == solution;
}

function verify() {
	if (currentTimeout)
		return;
	
	finishAnimateVisibility();
	var delayAnimation = false;
	var animationDelay = 150;
	if (displayingFullSolution) {
		setElementByIdVisible("solution", false);
		document.getElementById("answer").style.display = "block";
		document.getElementById("readOnlyAnswer").style.display = "none";
		document.getElementById("verifyButton").style.display = "block";
		document.getElementById("continueButton").style.display = "none";
		document.getElementById("mcqContinueButton").style.display = "none";
		document.getElementById("answer").focus();
		displayingFullSolution = false;
		prepareForNextQuestion();
		animationDelay = 250;
	} else {
		var answer = document.getElementById("answer").value;
		var correct = answerEquivalentToSolution(answer, solution);
		mark(correct);
		if (ProVocPlugIn)
			ProVocPlugIn.wordWasAnsweredCorrectly(correct);
		if (correct) {
			var exact = answerEqualToSolution(answer, solution);
			if (!exact) {
				document.getElementById("answer").value = solution;
				displayFullSolution();
			}
			if (!displayCommentIfNecessary() && exact) {
				prepareForNextQuestion();
				delayAnimation = true;
			}
		} else
			if (++attempts >= retries) {
				document.getElementById("solution").innerText = solution;
				setElementByIdVisible("solution", true);
				displayFullSolution();
				displayCommentIfNecessary();
			}
	}
	if (delayAnimation)
		currentTimeout = setTimeout("clearAnswer(); animateElementVisibility(150); currentTimeout = null;", 500);
	else
		animateElementVisibility(animationDelay);
}

function setMCQMarkAtPosition(id, index)
{
	document.getElementById(id).style.top = (fontSize == "small" ? -8 : -3) + index * 30 + "px";
}

function choice(n) {
	if (n < 0 && !displayingFullSolution && !delayingChoices || currentTimeout)
		return;
	var animationDelay = 150;
	if (delayingChoices) {
		document.getElementById("displayButton").style.display = "none";
		setElementByIdVisible("mcqChoices", true);
		delayingChoices = false;
	} else if (displayingFullSolution) {
		document.getElementById("continueButton").style.display = "none";
		document.getElementById("mcqContinueButton").style.display = "none";
		displayingFullSolution = false;
		prepareForNextQuestion();
		animationDelay = 250;
	} else {
		document.getElementById("mcqChoiceList").style.display = "none";
		document.getElementById("mcqDoneChoiceList").style.display = "block";
		document.getElementById("mcqMark").src = n == solution ? "Images/Correct.png" : "Images/Wrong.png";
		setMCQMarkAtPosition("mcqMark", n);
		setElementByIdVisible("mcqMark", true);
		if (n == solution) {
			if (!displayCommentIfNecessary())
				currentTimeout = setTimeout("prepareForNextQuestion(); animateElementVisibility(150); currentTimeout = null;", 700);
		} else {
			displayFullSolution();
			displayCommentIfNecessary();
		}
		if (ProVocPlugIn)
			ProVocPlugIn.wordWasAnsweredCorrectly(n == solution);
	}
	animateElementVisibility(animationDelay);
}

function displayClicked()
{
	choice(-1);
}

function fileChanged(elem)
{
	selectFileID(elem.value);
}

function lessonChanged(elem)
{
	setLessonID(elem.value);
}

function directionChanged(elem)
{
	if (direction != elem.value) {
		direction = elem.value;
		setPreferenceForKey(direction, "direction");
		updateUI();
		askQuestion(true);
	}
}

function commentDisplayChanged(elem)
{
	if (commentDisplay != elem.value) {
		commentDisplay = elem.value;
		setPreferenceForKey(commentDisplay, "commentDisplay");
		askQuestion(true);
	}
}

function modeChanged(elem)
{
	mode = elem.value;
	setPreferenceForKey(mode, "mode");
	updateModeParameterUI();
}

function retriesChanged(elem)
{
	retries = elem.value;
	setPreferenceForKey(retries, "retries");
}

function delayMCQChanged(elem)
{
	delayChoices = elem.checked;
	setPreferenceForKey(delayChoices, "delayChoices");
}

function delayMCQLabelClick(event)
{
	var elem = document.getElementById("delayMCQ");
	elem.checked = !elem.checked;
	delayMCQChanged(elem);
}

function speedChanged(elem)
{
	speed = elem.value;
	setPreferenceForKey(speed, "speed");
}

function fontSizeChanged(elem)
{
	fontSize = elem.value;
	setPreferenceForKey(fontSize, "fontSize");
}

function updateBackgroundColor()
{
	document.getElementById("background").src = color == "black" ? "Default.png" : "Skins/" + color + ".png";
}

function colorChanged(elem)
{
	color = elem;
	setPreferenceForKey(color, "color");
	updateBackgroundColor();
}

function updateStaticUI()
{
	updateBackgroundColor();
	
	document.getElementById("loading").innerText = getLocalizedString("Loading");
	document.getElementById("noFileMessage").innerText = getLocalizedString("No File");
	document.getElementById("notConfigured").innerText = getLocalizedString("Not Configured");
	document.getElementById("noWord").innerText = getLocalizedString("No Word");

	document.getElementById("fileLabel").innerText = getLocalizedString("File:");
	document.getElementById("lessonLabel").innerText = getLocalizedString("Lesson:");

	document.getElementById("commentLabel").innerText = getLocalizedString("Comment:");
	
	document.getElementById("directionLabel").innerText = getLocalizedString("Direction:");
	document.getElementById("directionPopup").value = direction;

	document.getElementById("commentDisplayLabel").innerText = getLocalizedString("Display Comments:");
	document.getElementById("commentDisplayPopup").options[0].innerText = getLocalizedString("Always");
	document.getElementById("commentDisplayPopup").options[1].innerText = getLocalizedString("After Answers");
	document.getElementById("commentDisplayPopup").options[4].innerText = getLocalizedString("Never");
	document.getElementById("commentDisplayPopup").value = commentDisplay;

	document.getElementById("modeLabel").innerText = getLocalizedString("Mode:");
	document.getElementById("modePopup").options[0].innerText = getLocalizedString("Questions");
	document.getElementById("modePopup").options[1].innerText = getLocalizedString("MCQ");
	document.getElementById("modePopup").options[2].innerText = getLocalizedString("Slideshow");
	document.getElementById("modePopup").value = mode;	

	document.getElementById("retriesLabel").innerText = getLocalizedString("Retries:");
	document.getElementById("retriesPopup").value = retries;	

	document.getElementById("delayMCQText").innerText = getLocalizedString("Delay Choices");
	document.getElementById("delayMCQ").checked = delayChoices;	

	document.getElementById("speedLabel").innerText = getLocalizedString("Speed:");
	document.getElementById("speedPopup").options[0].innerText = getLocalizedString("Manual");
	document.getElementById("speedPopup").options[1].innerText = getLocalizedString("Slow");
	document.getElementById("speedPopup").options[2].innerText = getLocalizedString("Medium");
	document.getElementById("speedPopup").options[3].innerText = getLocalizedString("Fast");
	document.getElementById("speedPopup").value = speed;	

	document.getElementById("fontSizeLabel").innerText = getLocalizedString("Text:");
	document.getElementById("fontSizePopup").options[0].innerText = getLocalizedString("Small");
	document.getElementById("fontSizePopup").options[1].innerText = getLocalizedString("Medium Size");
	document.getElementById("fontSizePopup").options[2].innerText = getLocalizedString("Large");
	document.getElementById("fontSizePopup").value = fontSize;

	document.getElementById("colorLabel").innerText = getLocalizedString("Color:");

	document.getElementById("downloadFiles").innerText = getLocalizedString("Download Files");
	document.getElementById("help").innerText = getLocalizedString("Help");
	document.getElementById("downloadMore").innerText = getLocalizedString("Download More");
	document.getElementById("copyright").innerText = getLocalizedString("Copyright");
	document.getElementById("update").innerText = getLocalizedString("New Version Available");
}

function updateUI()
{
	document.getElementById("directionPopup").options[0].innerText = getLocalizedString("Version");
	document.getElementById("directionPopup").options[1].innerText = getLocalizedString("Theme");
	document.getElementById("directionPopup").options[2].innerText = getLocalizedString("Both");

	document.getElementById("commentDisplayPopup").options[2].innerText = getLocalizedString("With SOURCE");
	document.getElementById("commentDisplayPopup").options[3].innerText = getLocalizedString("With TARGET");

	if (currentDirection == "version") {
		document.getElementById("questionLabel").innerText = getLocalizedString("SOURCE:");
		document.getElementById("answerLabel").innerText = getLocalizedString("TARGET:");
	} else {
		document.getElementById("questionLabel").innerText = getLocalizedString("TARGET:");
		document.getElementById("answerLabel").innerText = getLocalizedString("SOURCE:");
	}
}

function setup()
{
	loadPreferences();
	updateStaticUI();
	
	setElementByIdVisible("question", false);
	setElementByIdVisible("commentDiv", false);
	setElementByIdVisible("solution", false);
	setElementByIdVisible("cardQuestion", false);
	setElementByIdVisible("cardQuestion2", false);
	setElementByIdVisible("cardAnswer", false);
	setElementByIdVisible("cardAnswer2", false);
	setElementByIdVisible("cardComment", false);
	setElementByIdVisible("mcqMark", false);
	updateElementVisibility();
		
	updateUI();
	updateModeUI();
	document.addEventListener("keydown", keyPressed, true);
	document.getElementById("answer").focus();
	nextWord();
//	setTimeout("showPreferences();", 100);
	return 0;
}

function inputKeyPress(event)
{
	var c = event.charCode;
	if (c == 3 || c == 13)
		return false;
	else
		return true;
}

function keyPressed(event)
{
	if (event.metaKey) return;
	
	var handled = false;
	var c = event.charCode;
	if (!displayingPreferences)
		if (c == 3 || c == 13) {
			if (mode == "questions")
				verify();
			else if (mode == "mcq")
				choice(-1);
			else if (mode == "cards")
				performCardAction();
			handled = true;
		} else if (mode == "cards" && (c == 63235 || c == 63233)) {
			performCardAction();
			handled = true;
		} else {
			var s = String.fromCharCode(c);
			if (mode == "mcq") {
				var n = s - 1;
				if (n >= 0 && n < currentChoices)
					choice(n);
				handled = true;
			}
		}

	if (handled) {
		event.stopPropagation();
		event.preventDefault();
	}
}

function mouseClick(event)
{
	if (!displayingPreferences && mode == "cards")
		performCardAction();
}

function getLocalizedString(key)
{
	var s = key;
	try {
		s = localizedStrings[key];
	} catch (ex) {}

	s = s.replace(/SOURCE/g, sourceLanguage);
	s = s.replace(/sOURCE/g, sourceLanguage.toLowerCase());
	s = s.replace(/TARGET/g, targetLanguage);
	s = s.replace(/tARGET/g, targetLanguage.toLowerCase());

	return s;
}

function updateModeParameterUI()
{
	document.getElementById("questionParameters").style.display = mode == "questions" ? "block" : "none";
	document.getElementById("mcqParameters").style.display = mode == "mcq" ? "block" : "none";
	document.getElementById("cardParameters").style.display = mode == "cards" ? "block" : "none";
}

var files = null;
var dummy = 0;

function getFiles()
{
	if (ProVocPlugIn) {
		if (ProVocPlugIn.didFindNewFiles()) {
			files = new Array();
			var c = ProVocPlugIn.numberOfFiles();
			for (var i = 0; i < c; i++) {
				var name = ProVocPlugIn.nameOfFileAtIndex(i);
				var id = ProVocPlugIn.identifierOfFileAtIndex(i);
				files.push({name: name, id: id});
			}
		}
	} else {
		if (!files && dummy++ >= 2) {
			files = new Array();
			files.push({name: "None", id: "none"});
		}
	}
	return files;
}

function loadFiles()
{
	files = getFiles();	
	if (files) {
		populateFilePopup();
		selectFileID(fileID);
		document.getElementById("loading").style.display = "none";
		if (files.length > 0) {
			document.getElementById("ready").style.display = "block";
			document.getElementById("noFile").style.display = "none";
		} else {
			document.getElementById("ready").style.display = "none";
			document.getElementById("noFile").style.display = "block";
		}
	} else
		setTimeout("loadFiles();", 500);
}

var prevDirection = null;
var prevMode = null;
var prevDelayChoices = false;
var prevFileID = null;
var prefLessonID = null;

function checkForUpdates()
{
	if (ProVocPlugIn)
		if (ProVocPlugIn.isNewVersionAvailable())
			document.getElementById("update").style.display = "block";
}

function showPreferences(event)
{
	prevFileID = fileID;
	prevLessonID = lessonID;
	prevDirection = direction;
	prevMode = mode;
	prevDelayChoices = delayChoices;
	settingUp = false;
	
	loadFiles();
	
	if (mode == "cards") {
		cancelCardAction();
		setElementByIdVisible("cardQuestion", false);
		setElementByIdVisible("cardQuestion2", false);
		setElementByIdVisible("cardAnswer", false);
		setElementByIdVisible("cardAnswer2", false);
		setElementByIdVisible("cardComment", false);
		updateElementVisibility();
	}
	
	if (window.widget)
		widget.prepareForTransition("ToBack");
	
	document.getElementById("front").style.display = "none";
	updateModeParameterUI();
	document.getElementById("back").style.display = "block";
	displayingPreferences = true;
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);	

	setTimeout("checkForUpdates();", 100);
	document.getElementById('fliprollie').style.display = 'none';
	var elem = document.getElementById("retries")
	if (elem)
		elem.focus();
}

function verifyClicked()
{
	if (mode == "questions")
		verify();
	else if (mode == "mcq")
		choice(-1);
}

function checkWarning()
{
	var warning = null;
	if (!fileID)
		warning = "notConfigured";
	else if (emptyLesson())
		warning = "noWord";
	
	var names = new Array("notConfigured", "noWord");
	for (var i = 0; i < names.length; i++) {
		var name = names[i];
		document.getElementById(name).style.display = warning == name ? "block" : "none";
	}
	return warning != null;
}

function updateModeUI()
{
	if (checkWarning()) {
		document.getElementById("warning").style.display = "block";
		document.getElementById("questions").style.display = "none";
		document.getElementById("cards").style.display = "none";
		document.getElementById("displayButton").style.display = "none";
	} else {
		document.getElementById("warning").style.display = "none";
		document.getElementById("questions").style.display = mode == "questions" || mode == "mcq" ? "block" : "none";
		document.getElementById("cards").style.display = mode == "cards" ? "block" : "none";
		document.getElementById("displayButton").style.display = (mode == "mcq") && delayingChoices && !displayingFullSolution ? "block" : "none";
		document.getElementById("continueButton").style.display = (mode == "questions") && displayingFullSolution ? "block" : "none";
		document.getElementById("mcqContinueButton").style.display = (mode == "mcq") && !delayingChoices && displayingFullSolution ? "block" : "none";
	}
}

function doneClicked() {
	if (window.widget)
			widget.prepareForTransition("ToFront");

	updateFontSize();
	updateUI();
	document.getElementById("front").style.display = "block";
	document.getElementById("back").style.display = "none";
	displayingPreferences = false;
	
	if (prevFileID != fileID || prevLessonID != lessonID || prevDirection != direction || prevMode != mode || prevDelayChoices != delayChoices) {
		if (displayingFullSolution) {
			document.getElementById("solution").innerText = "";
			document.getElementById("answer").style.display = "block";
			document.getElementById("readOnlyAnswer").style.display = "none";
			displayingFullSolution = false;
		}
		nextWord();
	} else {
		if (mode == "questions")
			document.getElementById("answer").focus();
		else if (mode == "cards")
			showCardRecto(false);
	}

	updateModeUI();

	setTimeout("flipItBack();", 0);
}

function flipItBack()
{
	if (window.widget)
		setTimeout("widget.performTransition();", 0);
}

var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, element:null, timer:null};

function clamp (val, min, max)
{
    return val < min ? min : (val > max ? max : val);
}

function interpolation (from, to, ease)
{
    return from + (to - from) * ease;
}

function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
		
	
	T = clamp(time-animation.starttime, 0, animation.duration);
	
	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = interpolation (animation.from, animation.to, ease);
	}
	
	animation.element.style.opacity = animation.now;
}


function mousemove (event)
{
	if (!flipShown)
	{
		// fade in the flip widget
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13; // set it back one frame
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.element = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}

function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the flip widget
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13; // set it back one frame
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.element = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function enterflip(event)
{
	document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event)
{
	document.getElementById('fliprollie').style.display = 'none';
}

function show()
{
	if (wasHidden) {
		if (displayingPreferences)
			loadFiles();
		setFileID(fileID);
		setLessonID(lessonID);
		if (mode == "cards")
			nextWord();
		wasHidden = false;
	}
}

function hide()
{
	cancelCardAction();
	hideCardVerso();
	animateElementVisibility(1);
	wasHidden = true;
}

function removed()
{
}

function focused()
{
}

function blurred()
{
}

function remove()
{
	deletePreferences();
}

if (window.widget) {
	widget.onshow = show;
	widget.onhide = hide;
	window.onfocus = focused;
	window.onblur = blurred;
	window.onremove = remove;
}	

function filesFromDragEvent(event)
{
	var uri = null;

	try {
		uri = event.dataTransfer.getData("text/uri-list");
	} catch (ex) {
	}

	return uri;
}

function dragDrop(event)
{
	if (ProVocPlugIn) {
		var id = ProVocPlugIn.dropFiles(filesFromDragEvent(event));
		if (id) {
			fileID = id;
			loadFiles();
		}
	}

	event.stopPropagation();
	event.preventDefault();
}

function dragEnter(event)
{
	if (ProVocPlugIn)
		if (!ProVocPlugIn.canDropFiles(filesFromDragEvent(event)))
			return;
	event.stopPropagation();
	event.preventDefault();
}

function dragOver(event)
{
	if (ProVocPlugIn)
		if (!ProVocPlugIn.canDropFiles(filesFromDragEvent(event)))
			return;
	event.stopPropagation();
	event.preventDefault();
}

function dragLeave(event)
{
	event.stopPropagation();
	event.preventDefault();
}

function LOG__(message)
{
	if (window.widget)
		alert(message);
	if (window.console)
		window.console.log(message);
}
